function [res_J_opt,res_theta_opt] = SimulateSelfTuning(scenario,computationBudget,n_runs)
% Simulate self-tuning experiment.
%
% Input:
%   scenario            The considered scenario: imperfect model or imperfect
%                       (noisy) state measurements
%                       Choices:
%                           perturbedModel
%                           imperfectState
%   computationBudget   Computational budget; how many cost evaluations are
%                       we willing to spend? (# evaluations = 2*n_g*i_max)
%                       Choices:
%                           small (n_g=2, i_max=20)
%                           large (n_g=10, i_max=50)
%   n_runs              Number of self-tuning experiments performed.
%
% Output:
%   res_J_opt           Resulting cost
%   res_theta_opt       Resulting parameter
%   
% --
% Self-tuning LQR, Trimpe et al. IFAC World Congress 2014 paper
%
% Copyright:
% Sebastian Trimpe
% Max-Planck-Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [13.11.13]   first version to be published

if nargin<2
    n_runs = 1; % default: only once
end;


%% Configuration

% Optimization parameters
% Parameters independent of scenario.
specs.alpha = 0.602; 
specs.a_bar = 2;
specs.gamma = 0.101;
specs.c = 0.5;

% Parameter depending on scenario.
% Parameter a is chosen differently; it is recommended to be chosen
% relative to the gradient around the initial parameter value (see 
% reference [Spall 03], which is different in both cases.
switch scenario
    case 'perturbedModel'
        specs.a = 5.5e5;       
    case 'imperfectState'
        specs.a = 1.55e4;
    otherwise
        error('Invalid user input for SCENARIO.');
end;

% Computation budget: how many cost evaluations are we willing to spend?
switch computationBudget
    case 'small'
        specs.n_g = 2;
        specs.i_max = 20;
    case 'large'
        specs.n_g = 10;
        specs.i_max = 50;
    otherwise
        error('Invalid user input for COMPUTATIONBUDGET.');
end;


%% Load cost specification
cost = ConfigCost();


%% Optimization
fun = @(theta) EvaluateCost(theta,scenario);    % cost function
constr_proj = cost.constrProjFcn;               % constraint projection
theta_init = [1, 1];                            % initial guess

    
% Run self-tuning experiment
for i_run = 1:n_runs
        
    % Perform optimization/self-tuning experiment
    [theta_opt, J_opt, res_struct] = SPSA(fun, constr_proj, theta_init, specs);
    
    % Store results
    res_J_opt(i_run) = J_opt;
    res_theta_opt(i_run,:) = theta_opt;
    res_report(i_run).res_struct = res_struct;
    
end;



%% Save results
file_str = ['Data_SimulateSelfTuning_',scenario,'_',computationBudget,'_',num2str(n_runs)];
save(file_str,'res_J_opt','res_theta_opt','res_report');



